<?php

class HappyForms_Privacy_Settings {

	private static $instance;
	private static $hooked = false;

	public $save_action = 'happyforms_save_privacy_settings';
	public $save_nonce = 'happyforms-privacy-settings';
	public $option = 'happyforms_privacy_settings';
	public $schedule_cleanup = 'happyforms_schedule_privacy_cleanup';

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		if ( self::$hooked ) {
			return;
		}

		self::$hooked = true;

		add_action( $this->schedule_cleanup, array( $this, 'cleanup' ) );
		add_filter( 'happyforms_save_entries', array( $this, 'save_entries' ), 10, 2 );
		add_filter( 'happyforms_save_user_data', array( $this, 'save_user_data' ), 10, 2 );

		add_filter( 'happyforms_meta_fields', array( $this, 'meta_fields' ) );
		add_filter( 'happyforms_setup_controls', array( $this, 'setup_controls' ) );
		add_filter( 'happyforms_get_form_data', array( $this, 'transition_form_privacy_settings'), 99 );

		$this->schedule_cleanup();
	}

	//TODO remove once transition_form_privacy_settings has been removed.
	public function get_defaults() {
		$defaults = array(
			'save_submissions' => 1,
			'delete_submission_after' => 0,
			'delete_submission_days' => 60,
			'preserve_database_data' => 1,
			'save_user_data' => 1,
		);

		return $defaults;
	}

	//TODO remove once transition_form_privacy_settings has been removed.
	public function read( $as_array = false ) {
		$settings = get_option( $this->option, '' );
		$settings = wp_parse_args( $settings, $this->get_defaults() );

		return $settings;
	}

	public function save_entries( $save, $form ) {
		$save = $form['save_submissions'] == 1 ? true : false;

		return $save;
	}

	public function save_user_data( $save, $form ) {
		if ( ! $save ) {
			return $save;
		}

		$save = $form['save_user_data'] == 1 ? true : false;

		return $save;
	}

	public function schedule_cleanup() {
		if ( ! wp_next_scheduled( $this->schedule_cleanup ) ) {
			wp_schedule_event( time(), 'twicedaily', $this->schedule_cleanup );
		}
	}

	public function cleanup() {
		$form_controller = happyforms_get_form_controller();
		$forms = $form_controller->get();

		foreach( $forms as $form ) {
			if ( 0 === $form['delete_submission_after'] ) {
				continue;
			}

			$this->do_cleanup( $form );
		}
	}

	public function do_cleanup( $form ) {
		$date = '-' . $form['delete_submission_days'] . ' day';
		
		$args = array(
			'post_type' => happyforms_get_message_controller()->post_type,
			'post_status' => 'publish',
			'date_query' => array(
				'before' => $date,
			),
			'meta_key' => '_happyforms_form_id',
			'meta_value' => $form['ID'],
			'posts_per_page' => -1,
			'fields' => 'ids',
		);

		$args = apply_filters( 'happyforms_cleanup_activity_args', $args, $form );
		$post_ids = get_posts( $args );

		foreach ( $post_ids as $id ) {
			wp_delete_post( $id, true );
		}
	}

	public function get_fields() {
		$fields = array(
			'save_submissions' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'delete_submission_after' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'delete_submission_days' => array(
				'default' => 60,
				'sanitize' => 'happyforms_sanitize_float',
			),
			'save_user_data' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			// TODO delete once handler for global privacy settings has been complete removed.
			'per_form_privacy_settings' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
		);

		return $fields;
	}

	public function meta_fields( $fields ) {
		$fields = array_merge( $fields, $this->get_fields() );

		return $fields;
	}

	public function setup_controls( $controls ) {
		$setup_controls = array(
			5950 => array(
				'type' => 'checkbox',
				'label' => __( 'Store activity in database', 'happyforms' ),
				'field' => 'save_submissions',
			),
			5951 => array(
				'type' => 'checkbox',
				'label' => __( 'Delete activity after set number of days', 'happyforms' ),
				'field' => 'delete_submission_after',
			),
			5952 => array(
				'type' => 'group_start',
				'trigger' => 'delete_submission_after'
			),
			5953 => array(
				'type' => 'number',
				'label' => __( 'Number of days:', 'happyforms' ),
				'field' => 'delete_submission_days',
			),
			5954 => array(
				'type' => 'group_end'
			),
			5955 => array(
				'type' => 'checkbox',
				'label' => __( 'Capture user metadata (IP, language, platform, time and timezone)', 'happyforms' ),
				'field' => 'save_user_data',
			),
		);

		$controls = happyforms_safe_array_merge( $controls, $setup_controls );

		return $controls;
	}

	public function transition_form_privacy_settings( $form ) {
		if( 0 == $form['ID'] ) {
			$form['per_form_privacy_settings'] = 1;
			return $form;
		}

		if( 0 == $form['per_form_privacy_settings'] ) {
			$form['per_form_privacy_settings'] = 1;
			$settings = $this->read();

			if( 0 == $settings['save_submissions'] ) {
				$form['save_submissions'] = 0;
			}

			if( 1 == $settings['delete_submission_after'] ) {
				$form['delete_submission_after'] = 1;
				$form['delete_submission_days'] = $settings['delete_submission_days'];
			}

			if( 0 == $settings['save_user_data'] ) {
				$form['save_user_data'] = 0;
			}

		}

		return $form;
	}

}

if ( ! function_exists( 'happyforms_get_privacy_settings' ) ):

function happyforms_get_privacy_settings() {
	return HappyForms_Privacy_Settings::instance();
}

endif;

happyforms_get_privacy_settings();
